__author__ = 'Hari Jiang'


class Response(object):
    def __init__(self, status=None, headers=None, body=None):
        self.status = status
        self.headers = headers
        self.body = body

    @staticmethod
    def from_requests_response(response):
        return Response(status=response.status_code, body=response.text,
                        headers=dict(response.headers))

    def __str__(self):
        return """{status}
{headers}
{body}""".format(**self.__dict__)



__author__ = 'Hari Jiang'


class TestCaseError(RuntimeError):
    pass


class TestCaseMatcher(object):
    def __init__(self, ignore_headers=None):
        ignore_headers = ignore_headers or []
        self._ignore_headers = set(map(lambda h: h.upper(), ignore_headers))

    def _compare_status(self, r1, r2):
        return r1.status == r2.status

    def _compare_headers(self, expect, real):
        expect_headers = set({k.upper(): expect.headers[k] for k in expect.headers if
                              k.upper() not in self._ignore_headers}.items())
        real_headers = set({k.upper(): real.headers[k] for k in real.headers if
                            k.upper() not in self._ignore_headers}.items())
        return expect_headers == real_headers

    def _compare_body(self, r1, r2):
        return r1.body == r2.body

    def match_requests(self, expect, real):
        """
        compare requests
        :type expect: zerotest.response.Response
        :type real: zerotest.response.Response
        :return:
        """
        for attr in ('status', 'headers', 'body'):
            compare_func = '_compare_{}'.format(attr)
            match = getattr(self, compare_func)(expect, real)
            if not match:
                raise TestCaseError(
                    '{} not match, expect: {}, got: {}'.format(attr, getattr(expect, attr), getattr(real, attr)))



__author__ = 'Hari Jiang'

import argparse
import sys
from urlparse import urlparse
import os
import logging
import tempfile

from zerotest.common import init_logging_config

DESCRIPTION = """
Capture HTTP request/response and replay it for the test purpose.
"""
init_logging_config()

LOG = logging.getLogger(__name__)


class CLI(object):
    def __init__(self):
        self._parser = None
        self._parse_result = None

    def _init_arg_parser(self):
        parser = argparse.ArgumentParser(description=DESCRIPTION)
        subparsers = parser.add_subparsers(help='sub-command help', dest='subparser_name')

        server_parser = subparsers.add_parser('server', help='start zerotest local proxy server')
        server_parser.add_argument('url', help="target url: http://example.com")
        server_parser.add_argument('-f', '--file', help="file path to store record, default: [random path]")
        server_parser.add_argument('-b', '--bind', help="local bind address, default: 127.0.0.1")
        server_parser.add_argument('-p', '--port', help="local port, default: 7000")

        run_parser = subparsers.add_parser('replay', help='replay record file test')
        run_parser.add_argument('file', help="zerotest record file")
        run_parser.add_argument('--endpoint', help="replace requests endpoint, https://example.com")
        run_parser.add_argument('--ignore-headers', help="pass a list of ignored headers in response match", nargs='*')
        run_parser.add_argument('--verify-ssl', help="enable ssl verify", dest="verify_ssl", action="store_true")
        run_parser.add_argument('--no-verify-ssl', help="disable ssl verify", dest="verify_ssl",
                                action="store_false")
        run_parser.set_defaults(verify_ssl=False)

        self._parser = parser

    def run(self, argv=sys.argv[1:]):
        self._init_arg_parser()
        self._parse_result = self._parser.parse_args(argv)
        getattr(self, 'command_{}'.format(self._parse_result.subparser_name))()

    def exit_with_error_message(self, message):
        LOG.error(message)
        exit(1)

    def command_server(self):
        """
        sub-command start
        :return:
        """
        from zerotest.app import App

        forward_url = self._parse_result.url
        parsed_url = urlparse(forward_url)
        forward_host = parsed_url.hostname
        if not forward_host:
            self.exit_with_error_message("invalid url '{}'".format(forward_url))

        filepath = self._parse_result.file

        if not filepath:
            _, filepath = tempfile.mkstemp()
        else:
            if os.path.exists(filepath):
                LOG.warning("file '{}' is exists, new record will append to the file".format(filepath))

        app = App(forward_url, filepath)
        host = self._parse_result.bind or '127.0.0.1'
        port = int(self._parse_result.port or 7000)
        app.run(host, port)

    def command_replay(self):
        """
        sub-command replay
        run record file
        :return:
        """
        from zerotest.record.record_test_runner import RecordTestRunner

        filepath = self._parse_result.file
        if not os.path.exists(filepath):
            LOG.warning("file '{}' not exists".format(filepath))

        endpoint = self._parse_result.endpoint
        ignore_headers = self._parse_result.ignore_headers or []
        verify_ssl = self._parse_result.verify_ssl

        with open(filepath, 'r') as f:
            RecordTestRunner(f, endpoint=endpoint, ignore_headers=ignore_headers, verify_ssl=verify_ssl).run()


def main():
    CLI().run()


if __name__ == '__main__':
    main()



__author__ = 'Hari Jiang'


def dict_to_wsgi_headers(headers):
    wsgi_headers = []
    for k, v in headers.items():
        wsgi_headers.append((k, v))

    return wsgi_headers


def response_with_response(response, start_response):
    start_response("{} {}".format(response.status_code, response.reason), dict_to_wsgi_headers(response.headers))
    return response



__author__ = 'Hari Jiang'

from werkzeug.serving import run_simple

from zerotest.forwarder import Forwarder
from zerotest.record.http_recorder import HTTPRecorder
from zerotest.common import DEBUG


class App(object):
    def __init__(self, forward_url, record_file_path):
        forwarder = Forwarder(forward_url)
        self.recorder = HTTPRecorder(record_file_path)
        forwarder.on_forward_complete(lambda req, res: self.recorder.record_http(req, res))
        self._app = forwarder

    def run(self, host, port):
        self.recorder.start_service()
        try:
            run_simple(host, port, self._app, use_debugger=DEBUG, use_reloader=DEBUG)
        finally:
            self.recorder.close()



__author__ = 'Hari Jiang'
from urlparse import urlparse

import requests


class Request(object):
    def __init__(self, scheme=None, method=None, params=None, host=None, path=None, headers=None, data=None):
        self.scheme = scheme
        self.method = method
        self.headers = headers
        self.data = data
        self.params = params
        self.host = host
        self.path = path

    @property
    def endpoint(self):
        return "{scheme}://{host}".format(**self.__dict__)

    @endpoint.setter
    def endpoint(self, endpoint):
        parsed = urlparse(endpoint)
        self.scheme = parsed.scheme
        if parsed.port:
            host = "{}:{}".format(parsed.hostname, parsed.port)
        else:
            host = parsed.hostname
        self.host = host

    @property
    def url(self):
        return "{}{}".format(self.endpoint, self.path)

    def send_request(self, verify=True):
        return requests.request(self.method, self.url, headers=self.headers,
                                params=self.params, data=self.data,
                                stream=True, allow_redirects=False, verify=verify)

    def __str__(self):
        return """[{method}]{url}
{headers}
{data}""".format(method=self.method, url=self.url, headers=self.headers, data=self.data)



#!/usr/bin/env python

from setuptools import setup

setup(name='zerotest',
      version='0.0.0',
      description='Capture HTTP request/response and replay it for the test purpose',
      author='Hari Jiang',
      author_email='hari.jiang@outlook.com',
      license='MIT',
      packages=['zerotest'],
      install_requires=[
          'requests>=2.2.1',
          'Werkzeug>=0.10.4',
      ],
      entry_points={
          'console_scripts': [
              'zerotest=zerotest.cli:main',
          ],
      },
      )
